# ozi/filter.py
# SPDX-License-Identifier: Unlicense
"""Filters for use in the OZI project templates."""
from __future__ import annotations

import hashlib
import string
from datetime import datetime
from datetime import timezone
from functools import lru_cache

import requests
from packaging.version import parse

ALPHANUMS = string.ascii_uppercase + string.ascii_lowercase + '0123456789'


def current_date(_format: str) -> str:
    """Get the local date in a given format.

    :param _format: The date formatting template.
    :type _format: str
    :return: Formatted date
    :rtype: str
    """
    return datetime.now(tz=timezone.utc).date().strftime(_format)


@lru_cache
def underscorify(s: str) -> str:
    """Replace non-alphanumerics with underscores for normalization (cached).

    :param s: The text to be normalized
    :type s: str
    :return: The normalized text
    :rtype: str
    """
    return ''.join([c if c in ALPHANUMS else '_' for c in s])


@lru_cache
def wheel_repr(version: str) -> str:
    """Transform versions of the form "X.Y" into "pyXY".

    :param version: The version.
    :type version: str
    :return: The wheel tag version.
    :rtype: str
    """
    major, _, minor_end = version.partition('.')
    minor, *_ = minor_end.partition('.')
    return f'py{major}{minor}'


@lru_cache
def sha256sum(version: str) -> str:  # pragma: no cover
    """Filter to transform OZI source version into a hash of the tarball (cached).

    :param version: Version of OZI to get a hash for.
    :return: The corresponding SHA256 sum for the distribution tarball.
    :rtype: str
    """
    response = requests.get('https://pypi.org/pypi/OZI/json', timeout=30)
    match response.status_code:
        case 200:
            releases = response.json()['releases']
            latest_version = max(map(parse, releases.keys()))
            return str(
                [i for i in releases[latest_version] if i['filename'].endswith('.tar.gz')][
                    0
                ],
            )
        case _:
            checksum = hashlib.sha256()
            chunksize = 128 * 512
            response = requests.get(
                f'https://github.com/rjdbcm/OZI/archive/refs/tags/{version}.tar.gz',
                allow_redirects=True,
                stream=True,
                timeout=30,
            )
            for chunk in response.iter_content(chunksize):
                checksum.update(chunk)
            return checksum.hexdigest()


def next_minor(version: str) -> str:
    """Given a Python version, determine the next minor version.

    :param version: Python version string
    :type version: str
    :return: The next minor version
    :rtype: str
    """
    major, minor, *_ = map(int, version.split('.'))
    return f'{major}.{minor + 1}'



"""ozi-templates module"""

# Part of OZI-Templates.
# See LICENSE.txt in the project root for details.
from __future__ import annotations

from functools import _lru_cache_wrapper
from types import FunctionType
from typing import Any

from jinja2 import Environment
from jinja2 import PackageLoader
from jinja2 import select_autoescape

from ozi_templates.filter import current_date
from ozi_templates.filter import next_minor
from ozi_templates.filter import sha256sum
from ozi_templates.filter import underscorify
from ozi_templates.filter import wheel_repr

__all__ = ('load_environment',)
FILTERS = (
    next_minor,
    underscorify,
    zip,
    sha256sum,
    wheel_repr,
    current_date,
)


def _init_environment(_globals: dict[str, Any]) -> Environment:  # pragma: no cover
    """Initialize the rendering environment, set filters, and set global metadata."""
    env = Environment(
        loader=PackageLoader('ozi_templates', '.'),
        autoescape=select_autoescape(),
        enable_async=True,
        auto_reload=False,
    )
    for f in FILTERS:
        match f:
            case type():
                env.filters.setdefault(f.__name__, f)
            case FunctionType():
                env.filters.setdefault(f.__name__, f)
            case _lru_cache_wrapper():  # pragma: defer to pyright,mypy
                env.filters.setdefault(f.__wrapped__.__name__, f)
    env.globals = env.globals | _globals
    return env


def load_environment(  # pragma: no cover
    project: dict[str, str],
    _globals: dict[str, Any],
) -> Environment:
    """Load the rendering environment for templates.

    :return: jinja2 rendering environment for OZI
    :rtype: Environment
    """
    env = _init_environment(_globals)
    env.globals = env.globals | {'project': project}
    return env
